@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -43,7 +43,10 @@ import numbers as _numbers
 import re as _re
 from beartype.roar import BeartypeCallUnavailableTypeException
 from beartype._cave._caveabc import BoolType
-from beartype._util.py.utilpyversion import IS_PYTHON_AT_LEAST_3_9
+from beartype._util.py.utilpyversion import (
+    IS_PYTHON_AT_LEAST_3_9,
+    IS_PYTHON_AT_LEAST_3_8,
+)
 from collections import deque as _deque
 from collections.abc import (
     Collection as _Collection,
@@ -64,7 +67,6 @@ from enum import (
     EnumMeta as _EnumMeta,
 )
 from io import IOBase as _IOBase
-from sys import exc_info as _exc_info
 from typing import Any
 
 # Note that:
@@ -190,7 +192,7 @@ Depressingly, this type must now be manually redefined everywhere.
 '''
 
 
-NotImplementedType: type = type(NotImplemented)
+NotImplementedType: type = type(NotImplemented)  # type: ignore[misc]
 '''
 Type of the ``NotImplemented`` singleton.
 '''
@@ -216,6 +218,55 @@ Type of all **code objects** (i.e., C-based objects underlying all pure-Python
 callables to which those callables are compiled for efficiency).
 '''
 
+
+# If the active Python interpreter targets Python >= 3.8...
+if IS_PYTHON_AT_LEAST_3_8:
+    # Defer version-specific imports.
+    from types import CellType as _CellType  # type: ignore[attr-defined]
+
+    # Alias this type to this standard type.
+    #
+    # Note that this is explicitly required for "nuitka" support, which supports
+    # this standard type but *NOT* the non-standard approach used to deduce this
+    # type under Python 3.7 leveraged below.
+    ClosureVarCellType = _CellType
+    '''
+    Type of all **pure-Python closure cell variables.**
+    '''
+# Else, the active Python interpreter only targets Python 3.7. In this case...
+else:
+    def _closure_varcell_factory():
+        '''
+        Arbitrary function returning a closure-specific cell variable exposed by
+        an arbitrary closure isolated to this function.
+        '''
+
+        # Arbitrary outer local variable.
+        cell_variable = 42
+
+        def closure():
+            '''
+            Arbitrary closure exposing a closure-specific cell variable.
+            '''
+
+            nonlocal cell_variable
+
+        return closure.__closure__[0]  # pyright: ignore[reportOptionalSubscript]
+
+
+    # Although Python >= 3.7 now exposes an explicit closure cell variable type
+    # via the standard "types.CellType" object, this is of no benefit to older
+    # versions of Python. Ergo, the type of an arbitrary method wrapper
+    # guaranteed to *ALWAYS* exist is obtained instead.
+    ClosureVarCellType = type(_closure_varcell_factory())  # type: ignore[misc]
+    '''
+    Type of all **pure-Python closure cell variables.**
+    '''
+
+
+    # Delete this factory function for safety.
+    del _closure_varcell_factory
+
 # ....................{ TYPES ~ call : exception           }....................
 ExceptionTracebackType = _TracebackType
 '''
@@ -230,39 +281,6 @@ Type of all **call stack frame objects** (i.e., C-based objects
 encapsulating each call to each callable on the current call stack).
 '''
 
-# ....................{ TYPES ~ call : closure             }....................
-def _closure_varcell_factory():
-    '''
-    Arbitrary function returning a closure-specific cell variable exposed by an
-    arbitrary closure isolated to this function.
-    '''
-
-    # Arbitrary outer local variable.
-    cell_variable = 42
-
-    def closure():
-        '''
-        Arbitrary closure exposing a closure-specific cell variable.
-        '''
-
-        nonlocal cell_variable
-
-    return closure.__closure__[0]  # pyright: ignore[reportOptionalSubscript]
-
-
-# Although Python >= 3.7 now exposes an explicit closure cell variable type via
-# the standard "types.CellType" object, this is of no benefit to older versions
-# of Python. Ergo, the type of an arbitrary method wrapper guaranteed to
-# *ALWAYS* exist is obtained instead.
-ClosureVarCellType = type(_closure_varcell_factory())
-'''
-Type of all **pure-Python closure cell variables.**
-'''
-
-
-# Delete this factory function for safety.
-del _closure_varcell_factory
-
 # ....................{ TYPES ~ call : function            }....................
 FunctionType = _FunctionType
 '''
@@ -1215,7 +1233,7 @@ MethodUnboundTypes = (
 )
 '''
 Tuple of all **unbound method types** (i.e., types whose instances are callable
-objects bound to neither instances nor classes)
+objects bound to neither instances nor classes).
 
 Unbound decorator objects (e.g., non-callable instances of the builtin
 :class:`classmethod`, :class:`property`, or :class:`staticmethod` decorator
@@ -1306,6 +1324,19 @@ callable objects implemented in low-level C rather than high-level Python).
 '''
 
 
+CallablePyTypes = (
+    FunctionType,
+    MethodBoundInstanceOrClassType,
+)
+'''
+Tuple of all **pure-Python callable types** (i.e., types whose instances are
+callable objects implemented in high-level Python rather than low-level C).
+
+**This tuple is empty under PyPy,** which unconditionally compiles *all*
+pure-Python callables into C-based callables.
+'''
+
+
 CallableOrClassTypes = CallableTypes + (ClassType,)
 '''
 Tuple of all callable types as well as the type of all types.
