@@ -1,13 +1,13 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
-**Beartype PEP-compliant type hint exception raisers** (i.e., functions raising
-human-readable exceptions called by :mod:`beartype`-decorated callables on the
-first invalid parameter or return value failing a type-check against the
-PEP-compliant type hint annotating that parameter or return).
+**Beartype exception raisers** (i.e., high-level callables raising
+human-readable exceptions called by :func:`beartype.beartype`-decorated
+callables on the first invalid parameter or return value failing a type-check
+against the PEP-compliant type hint annotating that parameter or return).
 
 This private submodule is *not* intended for importation by downstream callers.
 '''
@@ -81,6 +81,7 @@ from beartype.typing import (
     Optional,
 )
 from beartype._cave._cavemap import NoneTypeOr
+from beartype._conf.confcls import BeartypeConf
 from beartype._data.hint.pep.sign.datapepsigncls import HintSign
 from beartype._data.hint.pep.sign.datapepsigns import (
     HintSignAnnotated,
@@ -96,45 +97,38 @@ from beartype._data.hint.pep.sign.datapepsignset import (
     HINT_SIGNS_ORIGIN_ISINSTANCEABLE,
     HINT_SIGNS_UNION,
 )
-from beartype._decor._error._errorsleuth import CauseSleuth
-from beartype._util.hint.utilhinttest import die_unless_hint
-from beartype._util.text.utiltextlabel import (
+from beartype._decor._error._errorcause import ViolationCause
+from beartype._decor._error._util.errorutilcolor import (
+    color_hint,
+    color_repr,
+    strip_text_ansi_if_configured,
+)
+from beartype._decor._error._util.errorutiltext import (
     prefix_callable_decorated_arg_value,
     prefix_callable_decorated_return_value,
 )
-from beartype._util.text.utiltextcolour import truth_colour, user_value_colour
+from beartype._util.hint.utilhinttest import die_unless_hint
 from beartype._util.text.utiltextmunge import suffix_unless_suffixed
 from beartype._util.text.utiltextrepr import represent_object
 from beartype._data.datatyping import TypeException
 
-# ....................{ MAPPINGS                           }....................
+# ....................{ GLOBALS                            }....................
 # Initialized with automated inspection below in the _init() function.
 PEP_HINT_SIGN_TO_GET_CAUSE_FUNC: Dict[
-    HintSign, Callable[[CauseSleuth], Optional[str]]] = {}
+    HintSign, Callable[[ViolationCause], ViolationCause]] = {}
 '''
 Dictionary mapping each **sign** (i.e., arbitrary object uniquely identifying a
 PEP-compliant type) to a private getter function defined by this submodule
-whose signature matches that of the :func:`_get_cause_or_none` function and
+whose signature matches that of the :func:`_find_cause` function and
 which is dynamically dispatched by that function to describe type-checking
-failures specific to that unsubscripted :mod:`typing` attribute.,
-'''
-
-# ....................{ CONSTANTS                          }....................
-# Assuming a line length of 80 characters, this magic number truncates
-# arbitrary object representations to 100 lines (i.e., 8000/80), which seems
-# more than reasonable and (possibly) not overly excessive.
-_CAUSE_TRIM_OBJECT_REPR_MAX_LEN = 8000
-'''
-Maximum length of arbitrary object representations suffixing human-readable
-strings returned by the :func:`_get_cause_or_none` getter function, intended to
-be sufficiently long to assist in identifying type-check failures but not so
-excessively long as to prevent human-readability.
+failures specific to that unsubscripted :mod:`typing` attribute.
 '''
 
 # ....................{ GETTERS                            }....................
 def get_beartype_violation(
     # Mandatory parameters.
     func: Callable,
+    conf: BeartypeConf,
     pith_name: str,
     pith_value: object,
 
@@ -186,6 +180,10 @@ def get_beartype_violation(
     ----------
     func : CallableTypes
         Decorated callable to raise this exception from.
+    conf : BeartypeConf
+        **Beartype configuration** (i.e., self-caching dataclass encapsulating
+        all flags, options, settings, and other metadata configuring the
+        current decoration of the decorated callable or class).
     pith_name : str
         Either:
 
@@ -194,7 +192,7 @@ def get_beartype_violation(
         * Else, the magic string ``return`` implying this object to be the
           value returned from this callable.
     pith_value : object
-        Passed parameter or returned value failing to satisfy this hint.
+        Passed parameter or returned value violating this hint.
     random_int: Optional[int]
         **Pseudo-random integer** (i.e., unsigned 32-bit integer
         pseudo-randomly generated by the parent :func:`beartype.beartype`
@@ -246,15 +244,19 @@ def get_beartype_violation(
           this type check when in fact this pith fails to do so.
     '''
     assert callable(func), f'{repr(func)} uncallable.'
+    assert isinstance(conf, BeartypeConf), f'{repr(conf)} not configuration.'
     assert isinstance(pith_name, str), f'{repr(pith_name)} not string.'
     assert isinstance(random_int, NoneTypeOr[int]), (
         f'{repr(random_int)} neither integer nor "None".')
+
     # print('''get_beartype_violation(
     #     func={!r},
+    #     conf={!r},
     #     pith_name={!r},
     #     pith_value={!r}',
-    # )'''.format(func, pith_name, pith_value))
+    # )'''.format(func, conf, pith_name, pith_value))
 
+    # ....................{ LOCALS                         }....................
     # Type of exception to be raised.
     exception_cls: TypeException = None  # type: ignore[assignment]
 
@@ -274,6 +276,7 @@ def get_beartype_violation(
         exception_prefix = prefix_callable_decorated_arg_value(
             func=func, arg_name=pith_name, arg_value=pith_value)
 
+    # ....................{ HINTS                          }....................
     # If this parameter or return value is unannotated, raise an exception.
     #
     # Note that this should *NEVER* occur, as the caller guarantees this
@@ -288,54 +291,82 @@ def get_beartype_violation(
     # PEP-compliant type hint annotating this parameter or return value.
     hint = func.__annotations__[pith_name]
 
-    # If this is *NOT* the PEP 484-compliant "typing.NoReturn" type hint
+    # If this hint is *NOT* the PEP 484-compliant "typing.NoReturn" type hint
     # permitted *ONLY* as a return annotation, this is a standard type hint
     # generally supported by both parameters and return values. In this case...
     if hint is not NoReturn:
-        # If type hint is *NOT* a supported type hint, raise an exception.
+        # If this hint is unsupported, raise an exception.
         die_unless_hint(hint=hint, exception_prefix=exception_prefix)
         # Else, this type hint is supported.
 
-    # Human-readable string describing the failure of this pith to satisfy this
-    # hint if this pith fails to satisfy this hint *OR* "None" otherwise (i.e.,
-    # if this pith satisfies this hint).
-    exception_cause = CauseSleuth(
+    # ....................{ CAUSE                          }....................
+    # Cause describing the failure of this pith to satisfy this hint.
+    violation_cause = ViolationCause(
         func=func,
+        conf=conf,
         pith=pith_value,
         hint=hint,
         cause_indent='',
         exception_prefix=exception_prefix,
         random_int=random_int,
-    ).get_cause_or_none()
+    ).find_cause()
 
     # If this pith satisfies this hint, *SOMETHING HAS GONE TERRIBLY AWRY.* In
     # theory, this should never happen, as the parent wrapper function
     # performing type checking should *ONLY* call this child helper function
     # when this pith does *NOT* satisfy this hint. In this case, raise an
     # exception encouraging the end user to submit an upstream issue with us.
-    if not exception_cause:
+    if not violation_cause.cause_str_or_none:
         pith_value_repr = represent_object(
             obj=pith_value, max_len=_CAUSE_TRIM_OBJECT_REPR_MAX_LEN)
         raise _BeartypeCallHintPepRaiseDesynchronizationException(
-            f'{exception_prefix}violates type hint {truth_colour(repr(hint))}, '
+            f'{exception_prefix}violates type hint {color_hint(repr(hint))}, '
             f'but utility function get_beartype_violation() '
             f'erroneously suggests this object satisfies this hint. '
             f'Please report this desynchronization failure to '
             f'the beartype issue tracker ({URL_ISSUES}) with '
             f'the accompanying exception traceback and '
             f'the representation of this object:\n'
-            f'{user_value_colour(pith_value_repr)}'
+            f'{color_repr(pith_value_repr)}'
         )
     # Else, this pith violates this hint as expected and as required for sanity.
 
     # This failure suffixed by a period if *NOT* yet suffixed by a period.
-    exception_cause_suffixed = suffix_unless_suffixed(
-        text=exception_cause, suffix='.')
+    violation_cause_suffixed = suffix_unless_suffixed(
+        text=violation_cause.cause_str_or_none, suffix='.')
 
+    # List of the one or more culprits responsible for this violation,
+    # initialized to the passed parameter or returned value violating this hint.
+    violation_culprits = [pith_value,]
+
+    # If the actual object directly responsible for this violation is *NOT* the
+    # passed parameter or returned value indirectly violating this hint, then
+    # the latter is almost certainly a container transitively containing the
+    # former as an item. In this case, add this item to this list as well.
+    if pith_value is not violation_cause.pith:
+        violation_culprits.append(violation_cause.pith)
+    # Else, the actual object directly responsible for this violation is the
+    # passed parameter or returned value indirectly violating this hint. In this
+    # case, avoid adding duplicate items to this list.
+
+    # ....................{ EXCEPTION                      }....................
+    # Exception message embedding this cause.
+    exception_message = (
+        f'{exception_prefix}violates type hint {color_hint(repr(hint))}, as '
+        f'{violation_cause_suffixed}'
+    )
+
+    #FIXME: Unit test us up, please.
+    # Strip all ANSI escape sequences from this message if requested by this
+    # external user-defined configuration.
+    exception_message = strip_text_ansi_if_configured(
+        text=exception_message, conf=conf)
+
+    #FIXME: Unit test that the caller receives the expected culprit, please.
     # Exception of the desired class embedding this cause.
     exception = exception_cls(  # type: ignore[misc]
-        f'{exception_prefix}violates type hint {truth_colour(repr(hint))}, as '
-        f'{exception_cause_suffixed}'
+        message=exception_message,
+        culprits=tuple(violation_culprits),
     )
 
     # Return this exception to the @beartype-generated type-checking wrapper
@@ -344,7 +375,19 @@ def get_beartype_violation(
     # this exception directly from that wrapper.
     return exception
 
-# ....................{ INITIALIZERS                       }....................
+# ....................{ PRIVATE ~ constants                }....................
+# Assuming a line length of 80 characters, this magic number truncates
+# arbitrary object representations to 100 lines (i.e., 8000/80), which seems
+# more than reasonable and (possibly) not overly excessive.
+_CAUSE_TRIM_OBJECT_REPR_MAX_LEN = 8000
+'''
+Maximum length of arbitrary object representations suffixing human-readable
+strings returned by the :func:`_find_cause` getter function, intended to
+be sufficiently long to assist in identifying type-check failures but not so
+excessively long as to prevent human-readability.
+'''
+
+# ....................{ PRIVATE ~ initializers             }....................
 def _init() -> None:
     '''
     Initialize this submodule.
@@ -352,53 +395,53 @@ def _init() -> None:
 
     # Defer heavyweight imports.
     from beartype._decor._error._errortype import (
-        get_cause_or_none_instance_type_forwardref,
-        get_cause_or_none_subclass_type,
-        get_cause_or_none_type_instance_origin,
+        find_cause_instance_type_forwardref,
+        find_cause_subclass_type,
+        find_cause_type_instance_origin,
     )
     from beartype._decor._error._pep._pep484._errornoreturn import (
-        get_cause_or_none_noreturn)
+        find_cause_noreturn)
     from beartype._decor._error._pep._pep484._errorunion import (
-        get_cause_or_none_union)
+        find_cause_union)
     from beartype._decor._error._pep._pep484585._errorgeneric import (
-        get_cause_or_none_generic)
+        find_cause_generic)
     from beartype._decor._error._pep._pep484585._errorsequence import (
-        get_cause_or_none_sequence_args_1,
-        get_cause_or_none_tuple,
+        find_cause_sequence_args_1,
+        find_cause_tuple,
     )
     from beartype._decor._error._pep._errorpep586 import (
-        get_cause_or_none_literal)
+        find_cause_literal)
     from beartype._decor._error._pep._errorpep593 import (
-        get_cause_or_none_annotated)
+        find_cause_annotated)
 
     # Map each originative sign to the appropriate getter *BEFORE* any other
     # mappings. This is merely a generalized fallback subsequently replaced by
     # sign-specific getters below.
     for pep_sign_origin_isinstanceable in HINT_SIGNS_ORIGIN_ISINSTANCEABLE:
         PEP_HINT_SIGN_TO_GET_CAUSE_FUNC[pep_sign_origin_isinstanceable] = (
-            get_cause_or_none_type_instance_origin)
+            find_cause_type_instance_origin)
 
     # Map each 1-argument sequence sign to its corresponding getter.
     for pep_sign_sequence_args_1 in HINT_SIGNS_SEQUENCE_ARGS_1:
         PEP_HINT_SIGN_TO_GET_CAUSE_FUNC[pep_sign_sequence_args_1] = (
-            get_cause_or_none_sequence_args_1)
+            find_cause_sequence_args_1)
 
     # Map each union-specific sign to its corresponding getter.
     for pep_sign_type_union in HINT_SIGNS_UNION:
         PEP_HINT_SIGN_TO_GET_CAUSE_FUNC[pep_sign_type_union] = (
-            get_cause_or_none_union)
+            find_cause_union)
 
     # Map each sign validated by a unique getter to that getter *AFTER* all
     # other mappings. These sign-specific getters are intended to replace all
     # other automated mappings above.
     PEP_HINT_SIGN_TO_GET_CAUSE_FUNC.update({
-        HintSignAnnotated: get_cause_or_none_annotated,
-        HintSignForwardRef: get_cause_or_none_instance_type_forwardref,
-        HintSignGeneric: get_cause_or_none_generic,
-        HintSignLiteral: get_cause_or_none_literal,
-        HintSignNoReturn: get_cause_or_none_noreturn,
-        HintSignTuple: get_cause_or_none_tuple,
-        HintSignType: get_cause_or_none_subclass_type,
+        HintSignAnnotated: find_cause_annotated,
+        HintSignForwardRef: find_cause_instance_type_forwardref,
+        HintSignGeneric: find_cause_generic,
+        HintSignLiteral: find_cause_literal,
+        HintSignNoReturn: find_cause_noreturn,
+        HintSignTuple: find_cause_tuple,
+        HintSignType: find_cause_subclass_type,
     })
 
 
