@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -25,13 +25,15 @@ from beartype.roar import (
     BeartypeDecorWrapperException,
     # BeartypeWarning,
 )
+from beartype.typing import no_type_check
 from beartype._cave._cavefast import (
     MethodDecoratorBuiltinTypes,
     MethodDecoratorClassType,
     MethodDecoratorPropertyType,
     MethodDecoratorStaticType,
 )
-from beartype._conf import BeartypeConf
+from beartype._conf.confcls import BeartypeConf
+from beartype._conf.confenum import BeartypeStrategy
 from beartype._data.datatyping import (
     BeartypeableT,
     TypeStack,
@@ -39,7 +41,7 @@ from beartype._data.datatyping import (
 )
 from beartype._data.cls.datacls import TYPES_BEARTYPEABLE
 from beartype._decor._wrapper.wrappermain import generate_code
-from beartype._decor._decorcall import BeartypeCall
+from beartype._check.checkcall import BeartypeCall
 from beartype._util.cache.pool.utilcachepoolobjecttyped import (
     acquire_object_typed,
     release_object_typed,
@@ -55,8 +57,6 @@ from traceback import format_exc
 from warnings import warn
 
 # ....................{ DECORATORS                         }....................
-#FIXME: Revise docstrings everywhere from "cls_root" and "cls_curr" to
-#"cls_stack", please.
 def beartype_object(
     # Mandatory parameters.
     obj: BeartypeableT,
@@ -564,27 +564,40 @@ def _beartype_func(
     #FIXME: Uncomment to display all annotations in "pytest" tracebacks.
     # func_hints = func.__annotations__
 
+    # If this configuration enables the no-time strategy performing *NO*
+    # type-checking, monkey-patch that callable with the standard
+    # @typing.no_type_check decorator detected above by the call to the
+    # is_func_unbeartypeable() tester on all subsequent decorations passed the
+    # same callable. (Doing so prevents all subsequent decorations from
+    # erroneously ignoring this previously applied no-time strategy.)
+    if conf.strategy is BeartypeStrategy.O0:
+        no_type_check(func)
+    # Else, this configuration enables a positive-time strategy performing at
+    # least the minimal amount of type-checking.
+
     # If that callable is unbeartypeable (i.e., if this decorator should
     # preserve that callable as is rather than wrap that callable with
     # constant-time type-checking), silently reduce to the identity decorator.
+    #
+    # Note that this conditional implicitly handles the prior conditional! :O
     if is_func_unbeartypeable(func):  # type: ignore[arg-type]
         return func  # type: ignore[return-value]
     # Else, that callable is beartypeable. Let's do this, folks.
 
-    # Previously cached callable metadata reinitialized from this callable.
+    # Previously cached callable metadata reinitialized from that callable.
     bear_call = acquire_object_typed(BeartypeCall)
     bear_call.reinit(func, conf, **kwargs)
 
     # Generate the raw string of Python statements implementing this wrapper.
     func_wrapper_code = generate_code(bear_call)
 
-    # If this callable requires *NO* type-checking, silently reduce to a noop
-    # and thus the identity decorator by returning this callable as is.
+    # If that callable requires *NO* type-checking, silently reduce to a noop
+    # and thus the identity decorator by returning that callable as is.
     if not func_wrapper_code:
         return func  # type: ignore[return-value]
-    # Else, this callable requires type-checking. Let's *REALLY* do this, fam.
+    # Else, that callable requires type-checking. Let's *REALLY* do this, fam.
 
-    # Function wrapping this callable with type-checking to be returned.
+    # Function wrapping that callable with type-checking to be returned.
     #
     # For efficiency, this wrapper accesses *ONLY* local rather than global
     # attributes. The latter incur a minor performance penalty, since local
@@ -705,7 +718,7 @@ def _beartype_type(
 
     # For the unqualified name and value of each direct (i.e., *NOT* indirectly
     # inherited) attribute of this class...
-    for attr_name, attr_value in cls.__dict__.items():
+    for attr_name, attr_value in cls.__dict__.items():  # pyright: ignore[reportGeneralTypeIssues]
         # If this attribute is beartypeable...
         if isinstance(attr_value, TYPES_BEARTYPEABLE):
             # This attribute decorated with type-checking configured by this
