@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -45,6 +45,7 @@ from beartype._util.hint.pep.proposal.utilpep585 import (
     is_hint_pep585_generic,
 )
 from beartype._util.py.utilpyversion import (
+    IS_PYTHON_AT_LEAST_3_11,
     IS_PYTHON_AT_MOST_3_9,
     IS_PYTHON_AT_LEAST_3_9,
 )
@@ -54,31 +55,34 @@ from beartype._data.datatyping import TupleTypes
 # If the active Python interpreter targets Python >= 3.9, implement this
 # getter to directly access the "__args__" dunder attribute.
 if IS_PYTHON_AT_LEAST_3_9:
-    _HINT_PEP585_ARGS_EMPTY_TUPLE = ((),)
-    '''
-    Tuple containing only the empty tuple, to be returned from the
-    :func:`get_hint_pep_args` getter when passed a :pep:`585`-compliant type
-    hint subscripted by the empty tuple (e.g., ``tuple[()]``).
-    '''
-
-
     def get_hint_pep_args(hint: object) -> tuple:
 
         # Return the value of the "__args__" dunder attribute if this hint
-        # defines this attribute *OR* the empty tuple otherwise.
-        hint_args = getattr(hint, '__args__', ())
+        # defines this attribute *OR* "None" otherwise.
+        hint_args = getattr(hint, '__args__', None)
 
-        # If this hint appears to be unsubscripted but is PEP 585-compliant,
-        # then this hint *WAS* actually subscripted by the empty tuple (e.g.,
-        # "tuple[()]"). For unknown reasons, the "HintGenericSubscriptedType"
-        # superclass erroneously reduces the empty tuple subscripting such hints
-        # to... literally nothing. Since doing so violates orthogonality with
-        # equivalent PEP 484-compliant type hints that do correctly preserve
-        # their arguments (e.g., "Tuple[()]"), silently coerce PEP 585-compliant
-        # type hints to behave like PEP 484-compliant type hints by returning a
-        # tuple containing only the empty tuple.
-        if not hint_args and isinstance(hint, HintGenericSubscriptedType):
-            return _HINT_PEP585_ARGS_EMPTY_TUPLE
+        # If this hint does *NOT* define this attribute, return the empty tuple.
+        if hint_args is None:
+            return ()
+        # Else, this hint defines this attribute.
+        #
+        # If this hint appears to be unsubscripted, then this hint *WAS*
+        # actually subscripted by the empty tuple (e.g., "tuple[()]",
+        # "typing.Tuple[()]"). Why? Because:
+        # * Python 3.11 made the unfortunate decision of ambiguously conflating
+        #   unsubscripted type hints (e.g., "tuple", "typing.Tuple") with type
+        #   hints subscripted by the empty tuple, preventing downstream
+        #   consumers from reliably distinguishing these two orthogonal cases.
+        # * Python 3.9 made a similar decision but constrained to only PEP
+        #   585-compliant empty tuple type hints (i.e., "tuple[()]"). PEP
+        #   484-compliant empty tuple type hints (i.e., "typing.Tuple[()]")
+        #   continued to correctly declare an "__args__" dunder attribute of
+        #   "((),)" until Python 3.11.
+        #
+        # Disambiguate these two cases on behalf of callers by returning a tuple
+        # containing only the empty tuple rather than returning the empty tuple.
+        elif not hint_args:
+            return _HINT_ARGS_EMPTY_TUPLE
         # Else, this hint is either subscripted *OR* is unsubscripted but not
         # PEP 585-compliant.
 
@@ -89,17 +93,20 @@ if IS_PYTHON_AT_LEAST_3_9:
 else:
     def get_hint_pep_args(hint: object) -> tuple:
 
-        # in python < 3.9, and unparametrized Generic would have the attribute
-        # `_special` set to True (but the actual __args__ are often a TypeVar).
-        # Because we want to differentiate between unparametrized and parametrized
-        # Generics, we check whether the hint is `_special` and if so, we return
-        # the empty tuple (instead of the TypeVar).
+        # Under python < 3.9, unparametrized generics have the attribute
+        # "_special" set to True despite the actual "__args__" typically being a
+        # "TypeVar" instance. Because we want to differentiate between
+        # unparametrized and parametrized generics, check whether the hint is
+        # "_special" and if so, we return the empty tuple instead of that
+        # "TypeVar" instance.
         if getattr(hint, '_special', False):
             return ()
+
         # Return the value of the "__args__" dunder attribute if this hint
         # defines this attribute *OR* the empty tuple otherwise.
         return getattr(hint, '__args__', ())
 
+
 # Document this function regardless of implementation details above.
 get_hint_pep_args.__doc__ = '''
     Tuple of all **typing arguments** (i.e., subscripted objects of the passed
@@ -575,8 +582,8 @@ def get_hint_pep_sign_or_none(hint: Any) -> Optional[HintSign]:
     #       False
     #
     # Ergo, the "typing.Generic" ABC uniquely identifies many but *NOT* all
-    # generics. While non-ideal, the failure of PEP 585-compliant generics
-    # to subclass a common superclass leaves us with little alternative.
+    # generics. While non-ideal, the failure of PEP 585-compliant generics to
+    # subclass a common superclass leaves us with little alternative.
     if is_hint_pep484585_generic(hint):
         return HintSignGeneric
     # Else, this hint is *NOT* a PEP 484- or 585-compliant generic.
@@ -785,3 +792,16 @@ def get_hint_pep_origin_type_isinstanceable_or_none(
         # Else, "None".
         None
     )
+
+# ....................{ PRIVATE ~ args                     }....................
+_HINT_ARGS_EMPTY_TUPLE = ((),)
+'''
+Tuple containing only the empty tuple, to be returned from the
+:func:`get_hint_pep_args` getter when passed either:
+
+* A :pep:`585`-compliant type hint subscripted by the empty tuple (e.g.,
+  ``tuple[()]``).
+* A :pep:`484`-compliant type hint subscripted by the empty tuple (e.g.,
+  ``typing.Tuple[()]``) under Python >= 3.11, which applied the :pep:`585`
+  approach throughout the :mod:`typing` module.
+'''
