@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -173,7 +173,7 @@ def is_hint_uncached(hint: object) -> bool:
 
     See Also
     ----------
-    :func:`beartype.util.hint.convert.utilconvcoerce.coerce_hint_any`
+    :func:`beartype._check.conv.convcoerce.coerce_hint_any`
         Further details.
     '''
 
