@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -53,15 +53,21 @@ Human-readable package name.
 '''
 
 
+LICENSE = 'MIT'
+'''
+Human-readable name of the license this package is licensed under.
+'''
+
+# ....................{ METADATA ~ package                 }....................
 PACKAGE_NAME = NAME.lower()
 '''
 Fully-qualified name of the top-level Python package containing this submodule.
 '''
 
 
-LICENSE = 'MIT'
+PACKAGE_TEST_NAME = f'{PACKAGE_NAME}_test'
 '''
-Human-readable name of the license this package is licensed under.
+Fully-qualified name of the top-level Python package exercising this project.
 '''
 
 # ....................{ PYTHON ~ version                   }....................
@@ -203,7 +209,7 @@ if _PYTHON_VERSION_PARTS < PYTHON_VERSION_MIN_PARTS:
 # For further details, see http://semver.org.
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
-VERSION = '0.11.0'
+VERSION = '0.12.0'
 '''
 Human-readable package version as a ``.``-delimited string.
 '''
@@ -241,7 +247,7 @@ attribution, see the top-level ``AUTHORS.md`` file.
 '''
 
 
-COPYRIGHT = '2014-2022 Beartype authors'
+COPYRIGHT = '2014-2023 Beartype authors'
 '''
 Legally binding copyright line excluding the license-specific prefix (e.g.,
 ``"Copyright (c)"``).
@@ -270,6 +276,12 @@ have yet to be stabilized and hence tagged. Hence, this URL is typically valid
 '''
 
 
+URL_FORUMS = f'{URL_HOMEPAGE}/discussions'
+'''
+URL of this package's user forums.
+'''
+
+
 URL_ISSUES = f'{URL_HOMEPAGE}/issues'
 '''
 URL of this package's issue tracker.
@@ -347,6 +359,15 @@ LIBS_TESTTIME_OPTIONAL = (
     #     https://mypy.readthedocs.io/en/stable/faq.html#does-it-run-on-pypy
     'mypy >=0.800; platform_python_implementation != "PyPy"',
 
+    #FIXME: Let's avoid attempting to remotely compile with nuitka under GitHub
+    #Actions-hosted continuous integration (CI) for the moment. Doing so is
+    #non-trivial enough under local testing workflows. *sigh*
+    # Require a reasonably recent version of nuitka if the current platform is a
+    # Linux distribution *AND* the active Python interpreter targets Python >=
+    # 3.8. For questionable reasons best ignored, nuitka fails to compile
+    # beartype under Python <= 3.7.
+    # 'nuitka >=1.2.6; sys_platform == "linux" and python_version >= "3.8.0"',
+
     #FIXME: Consider dropping the 'and platform_python_implementation != "PyPy"'
     #clause now that "tox.ini" installs NumPy wheels from a third-party vendor
     #explicitly supporting PyPy.
@@ -452,6 +473,38 @@ See Also
     Further details.
 '''
 
+# ....................{ METADATA ~ libs : theme            }....................
+SPHINX_THEME_NAME = 'furo'
+'''
+Name of the third-party Sphinx extension providing the custom HTML theme
+preferred by this documentation.
+
+Note that we selected this theme according to mostly objective (albeit
+ultimately subjective) heuristic criteria. In descending order of importance, we
+selected the theme with:
+
+#. The most frequent git commit history.
+#. The open issues and pull requests (PRs).
+#. The most GitHub stars as a crude proxy for aggregate rating.
+
+Furo handily bested all other themes across all three criteria. Furo is very
+well-maintained, frequently closes out open issues and merges open PRs, and
+sports the highest quantity of GitHub stars by an overwhelming margin.
+
+See Also
+----------
+https://github.com/pradyunsg/furo
+    Upstream GitHub-hosted repository.
+'''
+
+
+SPHINX_THEME_VERSION_MINIMUM = '2022.6.21'
+'''
+Human-readable minimum version as a ``.``-delimited string of the **Furo Sphinx
+theme** (i.e., :mod:`furo`) optionally leveraged when building package
+documentation.
+'''
+
 # ....................{ METADATA ~ libs : doc              }....................
 _LIB_DOCTIME_MANDATORY_VERSION_MINIMUM_SPHINX = '4.1.0'
 '''
@@ -465,16 +518,9 @@ Specifically, this project requires:
 '''
 
 
-_LIB_DOCTIME_MANDATORY_VERSION_MINIMUM_SPHINX_RTD_THEME = '0.5.1'
-'''
-Human-readable minimum version as a ``.``-delimited string of the **Read The
-Docs (RTD)-flavoured Sphinx theme** (i.e., :mod:`sphinx_rtd_theme`) optionally
-leveraged when building package documentation.
-'''
-
-
 LIBS_DOCTIME_MANDATORY = (
     f'sphinx >={_LIB_DOCTIME_MANDATORY_VERSION_MINIMUM_SPHINX}',
+    f'autoapi >=0.9.0',
 )
 '''
 **Mandatory developer documentation build-time package dependencies** (i.e.,
@@ -500,8 +546,7 @@ See Also
 LIBS_DOCTIME_MANDATORY_RTD = (
     f'sphinx =={_LIB_DOCTIME_MANDATORY_VERSION_MINIMUM_SPHINX}',
     (
-        f'sphinx-rtd-theme '
-        f'=={_LIB_DOCTIME_MANDATORY_VERSION_MINIMUM_SPHINX_RTD_THEME}'
+        f'{SPHINX_THEME_NAME} =={SPHINX_THEME_VERSION_MINIMUM}'
     ),
 )
 '''
