@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -85,11 +85,11 @@ this submodule rather than from :mod:`typing` directly: e.g.,
 # "import_typing_attr_or_none('Annotated')").
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 from beartype._util.py.utilpyversion import (
-    IS_PYTHON_AT_LEAST_3_7_2 as _IS_PYTHON_AT_LEAST_3_7_2,
-    IS_PYTHON_AT_LEAST_3_8   as _IS_PYTHON_AT_LEAST_3_8,
-    IS_PYTHON_AT_LEAST_3_9   as _IS_PYTHON_AT_LEAST_3_9,
-    IS_PYTHON_AT_LEAST_3_10  as _IS_PYTHON_AT_LEAST_3_10,
     IS_PYTHON_AT_LEAST_3_11  as _IS_PYTHON_AT_LEAST_3_11,
+    IS_PYTHON_AT_LEAST_3_10  as _IS_PYTHON_AT_LEAST_3_10,
+    IS_PYTHON_AT_LEAST_3_9   as _IS_PYTHON_AT_LEAST_3_9,
+    IS_PYTHON_AT_LEAST_3_8   as _IS_PYTHON_AT_LEAST_3_8,
+    IS_PYTHON_AT_LEAST_3_7_2 as _IS_PYTHON_AT_LEAST_3_7_2,
 )
 
 # ....................{ IMPORTS ~ all                      }....................
@@ -165,18 +165,24 @@ if _IS_PYTHON_AT_LEAST_3_8:
             is_typeddict as is_typeddict,  # pyright: ignore[reportGeneralTypeIssues]
         )
 
-        #FIXME: Uncomment once these PEPs go live:
-        # # If the active Python interpreter targets Python >= 3.11...
-        # if _IS_PYTHON_AT_LEAST_3_11:
-        #     from typing import (  # type: ignore[attr-defined]
-        #            LiteralString as LiteralString,
-        #            Self as Self,
-        #            TypeVarTuple as TypeVarTuple,
-        #            dataclass_transform,
-        #         #FIXME: Technically, this is already live -- but pyright insists
-        #         #it isn't. We don't particularly feel like fighting that.
-        #            reveal_type as reveal_type,
-        #     )
+        # If the active Python interpreter targets Python >= 3.11...
+        if _IS_PYTHON_AT_LEAST_3_11:
+            from typing import (  # type: ignore[attr-defined]
+                   LiteralString as LiteralString,  # pyright: ignore[reportGeneralTypeIssues]
+                   Never as Never,  # pyright: ignore[reportGeneralTypeIssues]
+                   NotRequired as NotRequired,  # pyright: ignore[reportGeneralTypeIssues]
+                   Required as Required,  # pyright: ignore[reportGeneralTypeIssues]
+                   Self as Self,  # pyright: ignore[reportGeneralTypeIssues]
+                   TypeVarTuple as TypeVarTuple,  # pyright: ignore[reportGeneralTypeIssues]
+                   Unpack as Unpack,  # pyright: ignore[reportGeneralTypeIssues]
+                   assert_never as assert_never,  # pyright: ignore[reportGeneralTypeIssues]
+                   assert_type as assert_type,  # pyright: ignore[reportGeneralTypeIssues]
+                   clear_overloads as clear_overloads,  # pyright: ignore[reportGeneralTypeIssues]
+                   dataclass_transform as dataclass_transform,  # pyright: ignore[reportGeneralTypeIssues]
+                   reveal_type as reveal_type,  # pyright: ignore[reportGeneralTypeIssues]
+                   get_overloads as get_overloads,  # pyright: ignore[reportGeneralTypeIssues]
+                   reveal_type as reveal_type,  # pyright: ignore[reportGeneralTypeIssues]
+            )
 
 # ....................{ PEP ~ 544                          }....................
 # If this interpreter is either performing static type-checking (e.g., via mypy)
