@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
-# --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# --------------------( LICENSE                            )--------------------
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -10,19 +10,19 @@ This submodule unit tests the subset of the public API of the :mod:`beartype`
 package defined by the private :mod:`beartype._conf` submodule.
 '''
 
-# ....................{ IMPORTS                           }....................
-#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+# ....................{ IMPORTS                            }....................
+#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 # WARNING: To raise human-readable test errors, avoid importing from
 # package-specific submodules at module scope.
-#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
-# ....................{ TESTS                             }....................
+# ....................{ TESTS                              }....................
 def test_api_conf_strategy() -> None:
     '''
     Test the public :func:`beartype.BeartypeStrategy` enumeration.
     '''
 
-    # Defer heavyweight imports.
+    # Defer test-specific imports.
     from beartype import BeartypeStrategy
 
     # Assert this enumeration declares the expected members.
@@ -32,25 +32,27 @@ def test_api_conf_strategy() -> None:
     assert isinstance(BeartypeStrategy.On, BeartypeStrategy)
 
 
-def test_api_conf_type() -> None:
+def test_api_conf_dataclass() -> None:
     '''
     Test the public :func:`beartype.BeartypeConf` class.
     '''
 
-    # Defer heavyweight imports.
+    # ....................{ IMPORTS                        }....................
+    # Defer test-specific imports.
     from beartype import BeartypeConf, BeartypeStrategy
     from beartype.roar import BeartypeConfException
     from pytest import raises
 
+    # ....................{ PASS                           }....................
     # Assert beartype configurations to be self-memoizing across both
     # unparametrized and parametrized type instantiations.
     #
-    # Note that the latter explicitly validates that the "strategy" parameter
-    # may be passed positionally.
+    # Note that the latter explicitly validates that this memoization ignores
+    # the order in which parameters are passed.
     assert BeartypeConf() is BeartypeConf()
     assert (
-        BeartypeConf(BeartypeStrategy.On, is_debug=True) is
-        BeartypeConf(BeartypeStrategy.On, is_debug=True)
+        BeartypeConf(strategy=BeartypeStrategy.On, is_debug=True, is_color=True, is_pep484_tower=True) is
+        BeartypeConf(is_pep484_tower=True, is_color=True, is_debug=True, strategy=BeartypeStrategy.On)
     )
 
     # Default (i.e., unparametrized) beartype configuration.
@@ -58,31 +60,33 @@ def test_api_conf_type() -> None:
 
     # Assert this configuration to default to the expected public fields.
     assert bear_conf_default.strategy is BeartypeStrategy.O1
+    assert bear_conf_default.is_color is None
+    assert bear_conf_default.is_pep484_tower is False
     assert bear_conf_default.is_debug is False
 
-    # Assert that attempting to modify any public raises the expected
-    # exception.
-    with raises(AttributeError):
-        bear_conf_default.strategy = BeartypeStrategy.O0
-    with raises(AttributeError):
-        bear_conf_default.is_debug = True
+    # All possible keyword arguments initialized to non-default values with
+    # which to instantiate a non-default beartype configuration.
+    bear_conf_nondefault_kwargs = dict(
+        is_color=True,
+        is_debug=True,
+        is_pep484_tower=True,
+        strategy=BeartypeStrategy.Ologn,
+    )
 
     # Arbitrary parametrized beartype configuration.
-    bear_conf_nondefault = BeartypeConf(
-        strategy=BeartypeStrategy.Ologn, is_debug=True)
-
-    # Assert two identical configurations to compare equal.
-    assert bear_conf_default == BeartypeConf()
-    assert bear_conf_nondefault == BeartypeConf(
-        strategy=BeartypeStrategy.Ologn, is_debug=True)
+    bear_conf_nondefault = BeartypeConf(**bear_conf_nondefault_kwargs)
 
     # Assert two differing configurations to compare unequal.
     assert bear_conf_default != bear_conf_nondefault
 
+    # Assert two identical configurations to compare equal.
+    assert bear_conf_default == BeartypeConf()
+    assert bear_conf_nondefault == BeartypeConf(**bear_conf_nondefault_kwargs)
+
     # Assert two identical configurations to hash equal.
     assert hash(bear_conf_default) == hash(BeartypeConf())
-    assert hash(bear_conf_nondefault) == hash(BeartypeConf(
-        strategy=BeartypeStrategy.Ologn, is_debug=True))
+    assert hash(bear_conf_nondefault) == hash(
+        BeartypeConf(**bear_conf_nondefault_kwargs))
 
     # Assert two differing configurations to hash unequal.
     assert hash(bear_conf_default) != hash(bear_conf_nondefault)
@@ -93,14 +97,33 @@ def test_api_conf_type() -> None:
     # Assert this representation to contain the names of this class and all
     # public fields of this class.
     assert 'BeartypeConf' in bear_conf_repr
-    assert 'strategy' in bear_conf_repr
+    assert 'is_color' in bear_conf_repr
     assert 'is_debug' in bear_conf_repr
+    assert 'is_pep484_tower' in bear_conf_repr
+    assert 'strategy' in bear_conf_repr
 
+    # ....................{ FAIL                           }....................
     # Assert that instantiating a configuration with invalid parameters raises
     # the expected exceptions.
     with raises(BeartypeConfException):
-        BeartypeConf(strategy=(
-            'By all, but which the wise, and great, and good'))
+        BeartypeConf(is_color=(
+            'And many sounds, and much of life and death.'))
     with raises(BeartypeConfException):
         BeartypeConf(is_debug=(
             'Interpret, or make felt, or deeply feel.'))
+    with raises(BeartypeConfException):
+        BeartypeConf(is_pep484_tower=(
+            'In the calm darkness of the moonless nights,'))
+    with raises(BeartypeConfException):
+        BeartypeConf(strategy=(
+            'By all, but which the wise, and great, and good'))
+
+    # Assert that attempting to modify any public raises the expected exception.
+    with raises(AttributeError):
+        bear_conf_default.is_color = True
+    with raises(AttributeError):
+        bear_conf_default.is_debug = True
+    with raises(AttributeError):
+        bear_conf_default.is_pep484_tower = True
+    with raises(AttributeError):
+        bear_conf_default.strategy = BeartypeStrategy.O0
