@@ -1,10 +1,10 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
-**Beartype Decidedly Object-Oriented Runtime-checking (DOOR) API object-oriened
+**Beartype Decidedly Object-Oriented Runtime-checking (DOOR) API object-oriented
 unit tests.**
 
 This submodule unit tests the subset of the public API of the public
@@ -16,95 +16,6 @@ This submodule unit tests the subset of the public API of the public
 # WARNING: To raise human-readable test errors, avoid importing from
 # package-specific submodules at module scope.
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-from pytest import fixture
-
-# ....................{ FIXTURES                           }....................
-@fixture(scope='session')
-def hint_equality_cases() -> 'Iterable[Tuple[object, object, bool]]':
-    '''
-    Session-scoped fixture returning an iterable of **hint equality cases**
-    (i.e., 3-tuples ``(hint_a, hint_b, is_equal)`` describing the equality
-    relations between two PEP-compliant type hints), efficiently cached across
-    all tests requiring this fixture.
-
-    This iterable is intentionally defined by the return of this fixture rather
-    than as a global constant of this submodule. Why? Because the former safely
-    defers all heavyweight imports required to define this iterable to the call
-    of the first unit test requiring this fixture, whereas the latter unsafely
-    performs those imports at pytest test collection time.
-
-    Returns
-    --------
-    Iterable[Tuple[object, object, bool]]
-        Iterable of one or more 3-tuples ``(hint_a, hint_b, is_equal)``,
-        where:
-
-        * ``hint_a`` is the PEP-compliant type hint to be passed as the first
-          parameter to the :meth:`beartype.door.TypeHint.__equals__` tester.
-        * ``hint_b`` is the PEP-compliant type hint to be passed as the second
-          parameter to the :meth:`beartype.door.TypeHint.__equals__` tester.
-        * ``is_equal`` is ``True`` only if these hints are equal according to
-          that tester.
-    '''
-
-    # ..................{ IMPORTS                            }..................
-    from beartype._util.py.utilpyversion import IS_PYTHON_AT_LEAST_3_9
-
-    # Intentionally import from "typing" rather than "beartype.typing" to
-    # guarantee PEP 484-compliant type hints.
-    from typing import (
-        Any,
-        List,
-        Tuple,
-        Union,
-    )
-
-    # ..................{ LISTS                              }..................
-    HINT_EQUALITY_CASES = [
-        (tuple, Tuple, True),
-        (list, list, True),
-        (list, List, True),
-        (list, List[Any], True),
-
-        #FIXME: *UGH.* This used to pass, but we're honestly not quite sure why.
-        #To get this to pass now, it looks like we'll need to override
-        #UnionTypeHint._make_args() to coerce its arguments into a set and then
-        #back into a tuple to destroy arbitrary user-defined orderings: e.g.,
-        #    class UnionTypeHint(...):
-        #        def _make_args(self) -> Tuple[object, ...]:
-        #            args_ordered = super()._make_args()
-        #            return tuple(set(args_ordered))
-
-        # (Union[int, str], Union[str, int], True),
-
-        (Union[int, str], Union[str, list], False),
-        (tuple, Tuple[Any, ...], True),
-    ]
-
-    # If the active Python interpreter targets Python >= 3.9 and thus supports
-    # both PEP 585 and 593...
-    if IS_PYTHON_AT_LEAST_3_9:
-        from beartype.typing import Annotated
-        from collections.abc import (
-            Awaitable as AwaitableABC,
-            Sequence as SequenceABC,
-        )
-
-        # Append cases exercising version-specific relations.
-        HINT_EQUALITY_CASES.extend((
-            # PEP 585-compliant type hints.
-            (tuple[str, ...], Tuple[str, ...], True),
-            (list[str], List[str], True),
-            (AwaitableABC[SequenceABC[int]], AwaitableABC[SequenceABC[int]], True),
-
-            # PEP 593-compliant type hints.
-            (Annotated[int, "hi"], Annotated[int, "hi"], True),
-            (Annotated[int, "hi"], Annotated[int, "low"], False),
-            (Annotated[int, "hi"], Annotated[int, "low"], False),
-        ))
-
-    # Return this mutable list coerced into an immutable tuple for safety.
-    return tuple(HINT_EQUALITY_CASES)
 
 # ....................{ TESTS ~ dunder ~ creation          }....................
 def test_door_typehint_new() -> None:
@@ -112,7 +23,8 @@ def test_door_typehint_new() -> None:
     Test the :meth:`beartype.door.TypeHint.__new__` factory method.
     '''
 
-    # Defer heavyweight imports.
+    # ....................{ IMPORTS                        }....................
+    # Defer test-specific imports.
     from beartype.door import TypeHint
     from beartype.roar import BeartypeDoorNonpepException
     from pytest import raises
@@ -124,6 +36,7 @@ def test_door_typehint_new() -> None:
         List,
     )
 
+    # ....................{ PASS                           }....................
     # Assert that recreating a type hint against identical input yields the same
     # previously memoized type hint.
     assert TypeHint(List[Any]) is TypeHint(List[Any])
@@ -142,6 +55,7 @@ def test_door_typehint_new() -> None:
     # yielding the same previously memoized type hint.
     assert TypeHint(TypeHint(int)) is TypeHint(int)
 
+    # ....................{ FAIL                           }....................
     # Assert this factory raises the expected exception when passed an object
     # that is *not* a PEP-compliant type hint.
     with raises(BeartypeDoorNonpepException):
@@ -157,15 +71,18 @@ def test_door_typehint_mapping() -> None:
     the kind of low-level type hint passed to that factory method.
     '''
 
-    # Defer heavyweight imports.
+    # ....................{ IMPORTS                        }....................
+    # Defer test-specific imports.
     from beartype.door import TypeHint
     from beartype_test.a00_unit.data.hint.util.data_hintmetacls import (
         HintPepMetadata)
     from beartype_test.a00_unit.data.hint.util.data_hintmetautil import (
         iter_hints_piths_meta)
 
+    # ....................{ ASSERTS                        }....................
     # For each predefined type hint and associated metadata...
     for hint_pith_meta in iter_hints_piths_meta():
+        # ....................{ METADATA                   }....................
         # Metadata describing this type hint.
         hint_meta = hint_pith_meta.hint_meta
 
@@ -192,6 +109,7 @@ def test_door_typehint_mapping() -> None:
         # Assert that this instance is of the expected subclass.
         assert isinstance(wrapper, hint_meta.typehint_cls)
 
+        # ....................{ PROPERTIES                 }....................
         # Assert that the type hint wrapped by this instance is the same hint.
         wrapper_hint = wrapper.hint
         # print(f'wrapper_hint: {repr(wrapper_hint), id(wrapper_hint), type(wrapper_hint)}')
@@ -215,18 +133,18 @@ def test_door_typehint_repr() -> None:
 
 # ....................{ TESTS ~ dunders : compare          }....................
 def test_door_typehint_equals(
-    hint_equality_cases: 'Iterable[Tuple[object, object, bool]]') -> None:
+    door_cases_equality: 'Iterable[Tuple[object, object, bool]]') -> None:
     '''
     Test the :meth:`beartype.door.TypeHint.__equals__` dunder method.
 
     Parameters
     ----------
-    hint_equality_cases : Iterable[Tuple[object, object, bool]]
+    door_cases_equality : Iterable[Tuple[object, object, bool]]
         Iterable of one or more 3-tuples ``(hint_a, hint_b, is_equal)``,
         declared by the :func:`hint_subhint_cases` fixture.
     '''
 
-    # Defer heavyweight imports.
+    # Defer test-specific imports.
     from beartype.door import TypeHint
 
     # Intentionally import from "typing" rather than "beartype.typing" to
@@ -240,11 +158,11 @@ def test_door_typehint_equals(
     # "hint_equality_cases" iterable.
     typehint_unequal = TypeHint(Generator[Union[list, str], str, None])
 
-    # Arbitrary non-hint object.
+    # Arbitrary non-hint object. Note that strings are valid type hints!
     nonhint = b'Of insects, beasts, and birds, becomes its spoil;'
 
     # For each equality relation to be tested...
-    for hint_a, hint_b, IS_EQUAL in hint_equality_cases:
+    for hint_a, hint_b, IS_EQUAL in door_cases_equality:
         # "TypeHint" instances encapsulating these hints.
         typehint_a = TypeHint(hint_a)
         typehint_b = TypeHint(hint_b)
@@ -255,12 +173,12 @@ def test_door_typehint_equals(
 
         # Assert this tester returns the expected boolean for each such hint and
         # another arbitrary hint guaranteed to be unequal to these hints. In
-        # other words, this performs a smoke test.
+        # other words, perform a smoke test.
         assert typehint_a != typehint_unequal
         assert typehint_b != typehint_unequal
 
         # Assert this tester returns the expected boolean for each such hint and
-        # an arbitrary non-hint. In other words, this performs a smoke test.
+        # an arbitrary non-hint. In other words, perform another smoke test.
         assert typehint_a != nonhint
         assert typehint_b != nonhint
 
@@ -401,26 +319,55 @@ def test_door_typehint_len():
     assert len(TypeHint(Union[int, str])) == 2
 
 # ....................{ TESTS ~ properties                 }....................
+#FIXME: Implement this up, please. We'll want to pay particular attention to
+#edge cases in those "TypeHint" implementations overriding the _make_args()
+#superclass method. In all likelihood, this will warrant yet another fixture.
+# def test_door_typehint_args() -> None:
+#     '''
+#     Test the read-only :meth:`beartype.door.TypeHint.args` property.
+#     '''
+#
+#     # Defer test-specific imports.
+#     from beartype.door import TypeHint
+#     from beartype.roar import BeartypeDoorException, BeartypeDoorNonpepException
+#     from beartype_test.a00_unit.data.hint.data_hint import HINTS_IGNORABLE
+#     from beartype_test.a00_unit.data.hint.pep.data_pep import HINTS_PEP_META
+#     from contextlib import suppress
+#
+#     # Assert this method:
+#     # * Accepts unignorable PEP-compliant type hints.
+#     # * Rejects ignorable PEP-compliant type hints.
+#     for hint_pep_meta in HINTS_PEP_META:
+#         #FIXME: Remove this suppression *AFTER* improving "TypeHint" to support
+#         #all currently unsupported type hints. Most of these will be
+#         #"BeartypeDoorNonpepException", but there are some covariant type hints
+#         #(e.g. numpy.dtype[+ScalarType]) that will raise a "not invariant"
+#         #exception in the "TypeVarTypeHint" subclass.
+#         with suppress(BeartypeDoorException):
+#             assert TypeHint(hint_pep_meta.hint).is_ignorable is (
+#                 hint_pep_meta.is_ignorable)
+
+
 def test_door_typehint_is_ignorable() -> None:
     '''
-    Test the :meth:`beartype.door.TypeHint.is_ignorable` tester.
+    Test the read-only :meth:`beartype.door.TypeHint.is_ignorable` property.
     '''
 
-    # Defer heavyweight imports.
+    # Defer test-specific imports.
     from beartype.door import TypeHint
     from beartype.roar import BeartypeDoorException, BeartypeDoorNonpepException
     from beartype_test.a00_unit.data.hint.data_hint import HINTS_IGNORABLE
     from beartype_test.a00_unit.data.hint.pep.data_pep import HINTS_PEP_META
     from contextlib import suppress
 
-    # Assert this method accepts ignorable type hints.
+    # Assert this property accepts ignorable type hints.
     for hint_ignorable in HINTS_IGNORABLE:
         #FIXME: Remove this suppression *AFTER* improving "TypeHint" to support
         #all currently unsupported type hints.
         with suppress(BeartypeDoorNonpepException):
             assert TypeHint(hint_ignorable).is_ignorable is True
 
-    # Assert this method:
+    # Assert this property:
     # * Accepts unignorable PEP-compliant type hints.
     # * Rejects ignorable PEP-compliant type hints.
     for hint_pep_meta in HINTS_PEP_META:
