@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2022 Beartype authors.
+# Copyright (c) 2014-2023 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -23,9 +23,13 @@ def test_make_check_code_decoration() -> None:
     function.
     '''
 
-    # Defer heavyweight imports.
+    # Defer test-specific imports.
     from beartype._check.expr.exprmake import make_check_expr
+    from beartype._conf.confcls import BEARTYPE_CONF_DEFAULT
 
     # Assert this function generates identical code for identical hints and is
     # thus cached via memoization.
-    assert make_check_expr(str) is make_check_expr(str)
+    assert (
+        make_check_expr(str, BEARTYPE_CONF_DEFAULT) is
+        make_check_expr(str, BEARTYPE_CONF_DEFAULT)
+    )
